@@ -18,10 +18,11 @@
 #include "realm-daemon.h"
 #include "realm-dbus-constants.h"
 #include "realm-diagnostics.h"
-#include "realm-discovery.h"
+#include "realm-disco.h"
 #include "realm-errors.h"
 #include "realm-kerberos.h"
 #include "realm-kerberos-membership.h"
+#include "realm-options.h"
 #include "realm-packages.h"
 #include "realm-provider.h"
 #include "realm-samba.h"
@@ -57,10 +58,6 @@ static const gchar *SAMBA_PACKAGES[] = {
 	NULL
 };
 
-const gchar *NO_PACKAGES[] = {
-	NULL
-};
-
 static void realm_samba_kerberos_membership_iface (RealmKerberosMembershipIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (RealmSamba, realm_samba, REALM_TYPE_KERBEROS,
@@ -77,7 +74,6 @@ static void
 realm_samba_constructed (GObject *obj)
 {
 	RealmKerberos *kerberos = REALM_KERBEROS (obj);
-	GVariant *supported;
 
 	G_OBJECT_CLASS (realm_samba_parent_class)->constructed (obj);
 
@@ -86,21 +82,6 @@ realm_samba_constructed (GObject *obj)
 	                            REALM_DBUS_OPTION_CLIENT_SOFTWARE, REALM_DBUS_IDENTIFIER_WINBIND,
 	                            NULL);
 
-	/*
-	 * Each line is a combination of owner and what kind of credentials are supported,
-	 * same for enroll/leave. We can't accept a ccache, because samba3 needs
-	 * to have credentials limited to RC4.
-	 */
-	supported = realm_kerberos_membership_build_supported (
-			REALM_KERBEROS_CREDENTIAL_PASSWORD, REALM_KERBEROS_OWNER_ADMIN,
-			REALM_KERBEROS_CREDENTIAL_PASSWORD, REALM_KERBEROS_OWNER_USER,
-			REALM_KERBEROS_CREDENTIAL_AUTOMATIC, REALM_KERBEROS_OWNER_NONE,
-			0);
-	g_variant_ref_sink (supported);
-	realm_kerberos_set_supported_join_creds (kerberos, supported);
-	realm_kerberos_set_supported_leave_creds (kerberos, supported);
-	g_variant_unref (supported);
-
 	realm_kerberos_set_suggested_admin (kerberos, "Administrator");
 	realm_kerberos_set_login_policy (kerberos, REALM_KERBEROS_ALLOW_ANY_LOGIN);
 	realm_kerberos_set_required_package_sets (kerberos, SAMBA_PACKAGES);
@@ -141,6 +122,11 @@ lookup_login_prefix (RealmSamba *self)
 	gchar *workgroup;
 	gchar *separator;
 
+	/* When using default, just have a direct login format */
+	if (realm_samba_config_get_boolean (self->config, REALM_SAMBA_CONFIG_GLOBAL,
+	                                    "winbind use default domain", FALSE))
+		return g_strdup ("");
+
 	workgroup = realm_ini_config_get (self->config, REALM_SAMBA_CONFIG_GLOBAL, "workgroup");
 	if (workgroup == NULL)
 		return NULL;
@@ -154,25 +140,20 @@ lookup_login_prefix (RealmSamba *self)
 
 typedef struct {
 	GDBusMethodInvocation *invocation;
-	gchar *ccache_file;
-	gchar *computer_ou;
-	gchar *realm_name;
-	gchar *user_name;
-	GBytes *password;
+	GVariant *options;
+	RealmDisco *disco;
+	RealmCredential *cred;
 } EnrollClosure;
 
 static void
 enroll_closure_free (gpointer data)
 {
 	EnrollClosure *enroll = data;
-	g_free (enroll->realm_name);
-	g_free (enroll->computer_ou);
-	g_free (enroll->user_name);
-	g_bytes_unref (enroll->password);
+	realm_disco_unref (enroll->disco);
+	g_variant_unref (enroll->options);
+	realm_credential_unref (enroll->cred);
 	g_object_unref (enroll->invocation);
-	if (enroll->ccache_file)
-		realm_keberos_ccache_delete_and_free (enroll->ccache_file);
-	g_slice_free (EnrollClosure, enroll);
+	g_free (enroll);
 }
 
 static void
@@ -180,15 +161,15 @@ on_winbind_done (GObject *source,
                  GAsyncResult *result,
                  gpointer user_data)
 {
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
+	GTask *task = G_TASK (user_data);
 	GError *error = NULL;
 
 	realm_samba_winbind_configure_finish (result, &error);
 	if (error != NULL)
-		g_simple_async_result_take_error (res, error);
-	g_simple_async_result_complete (res);
-
-	g_object_unref (res);
+		g_task_return_error (task, error);
+	else
+		g_task_return_boolean (task, TRUE);
+	g_object_unref (task);
 }
 
 static void
@@ -196,44 +177,39 @@ on_join_do_winbind (GObject *source,
                     GAsyncResult *result,
                     gpointer user_data)
 {
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	EnrollClosure *enroll = g_simple_async_result_get_op_res_gpointer (res);
-	RealmSamba *self = REALM_SAMBA (g_async_result_get_source_object (G_ASYNC_RESULT (res)));
+	GTask *task = G_TASK (user_data);
+	EnrollClosure *enroll = g_task_get_task_data (task);
+	RealmSamba *self = g_task_get_source_object (task);
 	GHashTable *settings = NULL;
 	GError *error = NULL;
-	const gchar *workgroup = NULL;
-
-	realm_samba_enroll_join_finish (result, &settings, &error);
-	if (error == NULL) {
-		workgroup = g_hash_table_lookup (settings, "workgroup");
-		if (workgroup == NULL) {
-			g_set_error (&error, REALM_ERROR, REALM_ERROR_INTERNAL,
-			             _("Failed to calculate domain workgroup"));
-		}
-	}
+	const gchar *name;
 
+	realm_samba_enroll_join_finish (result, &error);
 	if (error == NULL) {
 		realm_ini_config_change (self->config, REALM_SAMBA_CONFIG_GLOBAL, &error,
 		                         "security", "ads",
-		                         "realm", enroll->realm_name,
-		                         "workgroup", workgroup,
+		                         "realm", enroll->disco->kerberos_realm,
+		                         "workgroup", enroll->disco->workgroup,
 		                         "template homedir", realm_settings_string ("users", "default-home"),
 		                         "template shell", realm_settings_string ("users", "default-shell"),
+		                         "netbios name", enroll->disco->explicit_netbios,
+		                         "password server", enroll->disco->explicit_server,
+		                         "kerberos method", "system keytab",
 		                         NULL);
 	}
 
 	if (error == NULL) {
-		realm_samba_winbind_configure_async (self->config, enroll->invocation,
-		                                     on_winbind_done, g_object_ref (res));
+		name = realm_kerberos_get_name (REALM_KERBEROS (self));
+		realm_samba_winbind_configure_async (self->config, name,
+		                                     enroll->invocation,
+		                                     on_winbind_done, g_object_ref (task));
 	} else {
-		g_simple_async_result_take_error (res, error);
-		g_simple_async_result_complete (res);
+		g_task_return_error (task, error);
 	}
 
 	if (settings)
 		g_hash_table_unref (settings);
-	g_object_unref (self);
-	g_object_unref (res);
+	g_object_unref (task);
 }
 
 static void
@@ -241,22 +217,21 @@ on_install_do_join (GObject *source,
                     GAsyncResult *result,
                     gpointer user_data)
 {
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	EnrollClosure *enroll = g_simple_async_result_get_op_res_gpointer (res);
+	GTask *task = G_TASK (user_data);
+	EnrollClosure *enroll = g_task_get_task_data (task);
 	GError *error = NULL;
 
 	realm_packages_install_finish (result, &error);
 	if (error == NULL) {
-		realm_samba_enroll_join_async (enroll->realm_name, enroll->user_name, enroll->password,
-		                               enroll->computer_ou, enroll->invocation,
-		                               on_join_do_winbind, g_object_ref (res));
+		realm_samba_enroll_join_async (enroll->disco, enroll->cred, enroll->options,
+		                               enroll->invocation, on_join_do_winbind,
+		                               g_object_ref (task));
 
 	} else {
-		g_simple_async_result_take_error (res, error);
-		g_simple_async_result_complete (res);
+		g_task_return_error (task, error);
 	}
 
-	g_object_unref (res);
+	g_object_unref (task);
 }
 
 static gboolean
@@ -278,72 +253,58 @@ validate_membership_options (GVariant *options,
 }
 
 static void
-realm_samba_enroll_async (RealmKerberosMembership *membership,
-                          const gchar *name,
-                          GBytes *password,
-                          RealmKerberosFlags flags,
-                          GVariant *options,
-                          GDBusMethodInvocation *invocation,
-                          GAsyncReadyCallback callback,
-                          gpointer user_data)
+realm_samba_join_async (RealmKerberosMembership *membership,
+                        RealmCredential *cred,
+                        GVariant *options,
+                        GDBusMethodInvocation *invocation,
+                        GAsyncReadyCallback callback,
+                        gpointer user_data)
 {
 	RealmKerberos *realm = REALM_KERBEROS (membership);
 	RealmSamba *self = REALM_SAMBA (realm);
-	GSimpleAsyncResult *res;
+	GTask *task;
 	EnrollClosure *enroll;
-	const gchar **packages;
 	GError *error = NULL;
 	gchar *enrolled;
 
-	res = g_simple_async_result_new (G_OBJECT (realm), callback, user_data,
-	                                 realm_samba_enroll_async);
-	enroll = g_slice_new0 (EnrollClosure);
-	enroll->realm_name = g_strdup (realm_kerberos_get_realm_name (realm));
+	task = g_task_new (realm, NULL, callback, user_data);
+	enroll = g_new0 (EnrollClosure, 1);
+	enroll->disco = realm_disco_ref (realm_kerberos_get_disco (realm));
 	enroll->invocation = g_object_ref (invocation);
-	enroll->computer_ou = realm_kerberos_calculate_join_computer_ou (realm, options);
-	enroll->user_name = g_strdup (name);
-	enroll->password = g_bytes_ref (password);
-	g_simple_async_result_set_op_res_gpointer (res, enroll, enroll_closure_free);
+	enroll->options = g_variant_ref (options);
+	enroll->cred = realm_credential_ref (cred);
+	g_task_set_task_data (task, enroll, enroll_closure_free);
 
 	/* Make sure not already enrolled in a realm */
 	enrolled = lookup_enrolled_realm (self);
 	if (enrolled != NULL) {
-		g_simple_async_result_set_error (res, REALM_ERROR, REALM_ERROR_ALREADY_CONFIGURED,
-		                                 _("Already joined to a domain"));
-		g_simple_async_result_complete_in_idle (res);
+		g_task_return_new_error (task, REALM_ERROR, REALM_ERROR_ALREADY_CONFIGURED,
+		                         _("Already joined to a domain"));
 
 	} else if (!validate_membership_options (options, &error)) {
-		g_simple_async_result_take_error (res, error);
-		g_simple_async_result_complete_in_idle (res);
+		g_task_return_error (task, error);
 
 	} else {
-		if (flags & REALM_KERBEROS_ASSUME_PACKAGES)
-			packages = NO_PACKAGES;
-		else
-			packages = SAMBA_PACKAGES;
-		realm_packages_install_async (packages, enroll->invocation,
-		                              on_install_do_join, g_object_ref (res));
+		realm_packages_install_async (SAMBA_PACKAGES, enroll->invocation, options,
+		                              on_install_do_join, g_object_ref (task));
 	}
 
 	g_free (enrolled);
-	g_object_unref (res);
+	g_object_unref (task);
 }
 
 typedef struct {
 	GDBusMethodInvocation *invocation;
-	gchar *realm_name;
-	gchar *ccache_file;
+	RealmDisco *disco;
 } LeaveClosure;
 
 static void
 leave_closure_free (gpointer data)
 {
 	LeaveClosure *leave = data;
-	g_free (leave->realm_name);
-	if (leave->ccache_file)
-		realm_keberos_ccache_delete_and_free (leave->ccache_file);
+	realm_disco_unref (leave->disco);
 	g_object_unref (leave->invocation);
-	g_slice_free (LeaveClosure, leave);
+	g_free (leave);
 }
 
 static void
@@ -351,31 +312,31 @@ on_deconfigure_done (GObject *source,
                      GAsyncResult *result,
                      gpointer user_data)
 {
-	GSimpleAsyncResult *async = G_SIMPLE_ASYNC_RESULT (user_data);
+	GTask *task = G_TASK (user_data);
 	GError *error = NULL;
 
 	realm_samba_winbind_deconfigure_finish (result, &error);
 	if (error != NULL)
-		g_simple_async_result_take_error (async, error);
-	g_simple_async_result_complete (async);
-	g_object_unref (async);
+		g_task_return_error (task, error);
+	else
+		g_task_return_boolean (task, TRUE);
+	g_object_unref (task);
 }
 
 static void
 leave_deconfigure_begin (RealmSamba *self,
-                         GSimpleAsyncResult *async)
+                         GTask *task)
 {
 	LeaveClosure *leave;
 	GError *error = NULL;
 
-	leave = g_simple_async_result_get_op_res_gpointer (async);
+	leave = g_task_get_task_data (task);
 
 	/* Flush the keytab of all the entries for this realm */
 	realm_diagnostics_info (leave->invocation, "Removing entries from keytab for realm");
 
-	if (!realm_kerberos_flush_keytab (leave->realm_name, &error)) {
-		g_simple_async_result_take_error (async, error);
-		g_simple_async_result_complete_in_idle (async);
+	if (!realm_kerberos_flush_keytab (leave->disco->kerberos_realm, &error)) {
+		g_task_return_error (task, error);
 		return;
 	}
 
@@ -386,14 +347,13 @@ leave_deconfigure_begin (RealmSamba *self,
 	                              "realm", NULL,
 	                              "security", "user",
 	                              NULL)) {
-		g_simple_async_result_take_error (async, error);
-		g_simple_async_result_complete_in_idle (async);
+		g_task_return_error (task, error);
 		return;
 	}
 
 	/* And then deconfigure winbind */
 	realm_samba_winbind_deconfigure_async (self->config, leave->invocation,
-	                                       on_deconfigure_done, g_object_ref (async));
+	                                       on_deconfigure_done, g_object_ref (task));
 }
 
 static void
@@ -401,9 +361,9 @@ on_leave_do_deconfigure (GObject *source,
                          GAsyncResult *result,
                          gpointer user_data)
 {
-	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
-	LeaveClosure *leave = g_simple_async_result_get_op_res_gpointer (res);
-	RealmSamba *self = REALM_SAMBA (g_async_result_get_source_object (user_data));
+	GTask *task = G_TASK (user_data);
+	LeaveClosure *leave = g_task_get_task_data (task);
+	RealmSamba *self = g_task_get_source_object (task);
 	GError *error = NULL;
 
 	/* We don't care if we can leave or not, just continue with other steps */
@@ -413,88 +373,56 @@ on_leave_do_deconfigure (GObject *source,
 		g_error_free (error);
 	}
 
-	leave_deconfigure_begin (self, res);
+	leave_deconfigure_begin (self, task);
 
-	g_object_unref (self);
-	g_object_unref (res);
+	g_object_unref (task);
 }
 
-static GSimpleAsyncResult *
-setup_leave (RealmSamba *self,
-             GVariant *options,
-             GDBusMethodInvocation *invocation,
-             GAsyncReadyCallback callback,
-             gpointer user_data)
+static void
+realm_samba_leave_async (RealmKerberosMembership *membership,
+                         RealmCredential *cred,
+                         GVariant *options,
+                         GDBusMethodInvocation *invocation,
+                         GAsyncReadyCallback callback,
+                         gpointer user_data)
 {
+	RealmSamba *self = REALM_SAMBA (membership);
+	RealmKerberos *kerberos = REALM_KERBEROS (self);
+	GTask *task;
 	LeaveClosure *leave;
-	GSimpleAsyncResult *async;
 	const gchar *realm_name;
 	gchar *enrolled;
 
-	realm_name = realm_kerberos_get_realm_name (REALM_KERBEROS (self));
+	realm_name = realm_kerberos_get_realm_name (kerberos);
 
-	async = g_simple_async_result_new (G_OBJECT (self), callback, user_data, setup_leave);
-	leave = g_slice_new0 (LeaveClosure);
-	leave->realm_name = g_strdup (realm_name);
+	task = g_task_new (self, NULL, callback, user_data);
+	leave = g_new0 (LeaveClosure, 1);
+	leave->disco = realm_disco_ref (realm_kerberos_get_disco (kerberos));
 	leave->invocation = g_object_ref (invocation);
-	g_simple_async_result_set_op_res_gpointer (async, leave, leave_closure_free);
+	g_task_set_task_data (task, leave, leave_closure_free);
 
 	/* Check that enrolled in this realm */
 	enrolled = lookup_enrolled_realm (self);
 	if (g_strcmp0 (enrolled, realm_name) != 0) {
-		g_simple_async_result_set_error (async, REALM_ERROR, REALM_ERROR_NOT_CONFIGURED,
-		                                 _("Not currently joined to this domain"));
-		g_simple_async_result_complete_in_idle (async);
-		g_object_unref (async);
-		return NULL;
-
+		g_task_return_new_error (task, REALM_ERROR, REALM_ERROR_NOT_CONFIGURED,
+		                         _("Not currently joined to this domain"));
+		g_object_unref (task);
+		return;
 	}
 
-	return async;
-}
+	switch (cred->type) {
+	case REALM_CREDENTIAL_PASSWORD:
+		realm_samba_enroll_leave_async (leave->disco, cred, options, leave->invocation,
+		                                on_leave_do_deconfigure, g_object_ref (task));
+		break;
+	case REALM_CREDENTIAL_AUTOMATIC:
+		leave_deconfigure_begin (self, task);
+		break;
+	default:
+		g_return_if_reached ();
+	}
 
-static void
-realm_samba_leave_password_async (RealmKerberosMembership *membership,
-                                  const gchar *name,
-                                  GBytes *password,
-                                  RealmKerberosFlags flags,
-                                  GVariant *options,
-                                  GDBusMethodInvocation *invocation,
-                                  GAsyncReadyCallback callback,
-                                  gpointer user_data)
-{
-	RealmSamba *self = REALM_SAMBA (membership);
-	GSimpleAsyncResult *async;
-	LeaveClosure *leave;
-
-	async = setup_leave (self, options, invocation, callback, user_data);
-	if (async == NULL)
-		return;
-
-	leave = g_simple_async_result_get_op_res_gpointer (async);
-	realm_samba_enroll_leave_async (leave->realm_name, name, password,
-	                                leave->invocation, on_leave_do_deconfigure,
-	                                g_object_ref (async));
-	g_object_unref (async);
-}
-
-static void
-realm_samba_leave_automatic_async (RealmKerberosMembership *membership,
-                                   RealmKerberosFlags flags,
-                                   GVariant *options,
-                                   GDBusMethodInvocation *invocation,
-                                   GAsyncReadyCallback callback,
-                                   gpointer user_data)
-{
-	RealmSamba *self = REALM_SAMBA (membership);
-	GSimpleAsyncResult *async;
-
-	async = setup_leave (self, options, invocation, callback, user_data);
-	if (async == NULL)
-		return;
-
-	leave_deconfigure_begin (self, async);
-	g_object_unref (async);
+	g_object_unref (task);
 }
 
 static gboolean
@@ -545,28 +473,33 @@ realm_samba_logins_async (RealmKerberos *realm,
                           RealmKerberosLoginPolicy login_policy,
                           const gchar **add,
                           const gchar **remove,
+                          GVariant *options,
                           GAsyncReadyCallback callback,
                           gpointer user_data)
 {
-	GSimpleAsyncResult *async;
+	GTask *task;
 	GError *error = NULL;
 
-	async = g_simple_async_result_new (G_OBJECT (realm), callback, user_data,
-	                                   realm_samba_logins_async);
+	task = g_task_new (realm, NULL, callback, user_data);
+
+	if (login_policy == REALM_KERBEROS_ALLOW_REALM_LOGINS)
+		login_policy = REALM_KERBEROS_ALLOW_ANY_LOGIN;
 
 	/* Sadly we don't support this option */
 	if (login_policy != REALM_KERBEROS_ALLOW_ANY_LOGIN &&
 	    login_policy != REALM_KERBEROS_POLICY_NOT_SET) {
-		g_simple_async_result_set_error (async, G_DBUS_ERROR, G_DBUS_ERROR_NOT_SUPPORTED,
-		                                 _("The Samba provider cannot restrict permitted logins."));
+		g_task_return_new_error (task, G_DBUS_ERROR, G_DBUS_ERROR_NOT_SUPPORTED,
+		                         _("The Samba provider cannot restrict permitted logins."));
 
 	/* Make note of the permitted logins, so we can return them in the property */
 	} else if (!realm_samba_change_logins (realm, invocation, add, remove, &error)) {
-		g_simple_async_result_take_error (async, error);
+		g_task_return_error (task, error);
+
+	} else {
+		g_task_return_boolean (task, TRUE);
 	}
 
-	g_simple_async_result_complete_in_idle (async);
-	g_object_unref (async);
+	g_object_unref (task);
 }
 
 static void
@@ -575,6 +508,7 @@ update_properties (RealmSamba *self)
 	RealmKerberos *kerberos = REALM_KERBEROS (self);
 	GPtrArray *permitted;
 	gchar *login_formats[2] = { NULL, NULL };
+	gboolean configured;
 	const gchar *name;
 	gchar *domain;
 	gchar *realm;
@@ -592,7 +526,15 @@ update_properties (RealmSamba *self)
 	realm_kerberos_set_realm_name (kerberos, realm);
 	g_free (realm);
 
-	realm_kerberos_set_configured (kerberos, lookup_is_enrolled (self));
+	/*
+	 * Although samba domains do not do much management of the system or
+	 * pull that much policy, we cannot limit who can log in from the domain
+	 * and also cannot join more than one domain, so this we mark a
+	 * configured domain as one that manages the system.
+	 */
+	configured = lookup_is_enrolled (self);
+	realm_kerberos_set_configured (kerberos, configured);
+	realm_kerberos_set_manages_system (kerberos, configured);
 
 	/* Setup the workgroup property */
 	prefix = lookup_login_prefix (self);
@@ -627,7 +569,7 @@ realm_samba_membership_generic_finish (RealmKerberosMembership *realm,
                                        GAsyncResult *result,
                                        GError **error)
 {
-	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
+	if (!g_task_propagate_boolean (G_TASK (result), error))
 		return FALSE;
 
 	update_properties (REALM_SAMBA (realm));
@@ -639,7 +581,7 @@ realm_samba_generic_finish (RealmKerberos *realm,
                             GAsyncResult *result,
                             GError **error)
 {
-	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
+	if (!g_task_propagate_boolean (G_TASK (result), error))
 		return FALSE;
 
 	update_properties (REALM_SAMBA (realm));
@@ -691,6 +633,26 @@ realm_samba_finalize (GObject *obj)
 	G_OBJECT_CLASS (realm_samba_parent_class)->finalize (obj);
 }
 
+static void
+realm_samba_discover_myself (RealmKerberos *realm,
+                             RealmDisco *disco)
+{
+	RealmSamba *self = REALM_SAMBA (realm);
+	gchar *value;
+
+	value = realm_ini_config_get (self->config, REALM_SAMBA_CONFIG_GLOBAL, "workgroup");
+	g_free (disco->workgroup);
+	disco->workgroup = value;
+
+	value = realm_ini_config_get (self->config, REALM_SAMBA_CONFIG_GLOBAL, "netbios name");
+	g_free (disco->explicit_netbios);
+	disco->explicit_netbios = value;
+
+	value = realm_ini_config_get (self->config, REALM_SAMBA_CONFIG_GLOBAL, "password server");
+	g_free (disco->explicit_server);
+	disco->explicit_server = value;
+}
+
 void
 realm_samba_class_init (RealmSambaClass *klass)
 {
@@ -699,6 +661,7 @@ realm_samba_class_init (RealmSambaClass *klass)
 
 	kerberos_class->logins_async = realm_samba_logins_async;
 	kerberos_class->logins_finish = realm_samba_generic_finish;
+	kerberos_class->discover_myself = realm_samba_discover_myself;
 
 	object_class->constructed = realm_samba_constructed;
 	object_class->set_property = realm_samba_set_property;
@@ -711,11 +674,33 @@ realm_samba_class_init (RealmSambaClass *klass)
 static void
 realm_samba_kerberos_membership_iface (RealmKerberosMembershipIface *iface)
 {
-	iface->enroll_password_async = realm_samba_enroll_async;
-	iface->enroll_finish = realm_samba_membership_generic_finish;
-	iface->unenroll_password_async = realm_samba_leave_password_async;
-	iface->unenroll_automatic_async = realm_samba_leave_automatic_async;
-	iface->unenroll_finish = realm_samba_membership_generic_finish;
+	/*
+	 * Each line is a combination of owner and what kind of credentials are supported,
+	 * same for enroll/leave. We can't accept a ccache, because samba3 needs
+	 * to have credentials limited to RC4.
+	 */
+
+	static const RealmCredential join_supported[] = {
+		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_ADMIN },
+		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_USER },
+		{ REALM_CREDENTIAL_CCACHE, REALM_CREDENTIAL_OWNER_ADMIN },
+		{ 0, },
+	};
+
+	static const RealmCredential leave_supported[] = {
+		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_ADMIN },
+		{ REALM_CREDENTIAL_PASSWORD, REALM_CREDENTIAL_OWNER_USER },
+		{ REALM_CREDENTIAL_AUTOMATIC, REALM_CREDENTIAL_OWNER_NONE },
+		{ 0, },
+	};
+
+	iface->join_async = realm_samba_join_async;
+	iface->join_finish = realm_samba_membership_generic_finish;
+	iface->join_creds_supported = join_supported;
+
+	iface->leave_async = realm_samba_leave_async;
+	iface->leave_finish = realm_samba_membership_generic_finish;
+	iface->leave_creds_supported = leave_supported;
 }
 
 RealmKerberos *
