@@ -18,15 +18,16 @@
 #include "realm-daemon.h"
 #include "realm-dbus-constants.h"
 #include "realm-diagnostics.h"
-#include "realm-discovery.h"
 #include "realm-errors.h"
 #include "realm-packages.h"
 #include "realm-provider.h"
 #include "realm-service.h"
 #include "realm-sssd.h"
 #include "realm-sssd-config.h"
+#include "safe-printf.h"
 
 #include <glib/gstdio.h>
+#include <glib/gi18n.h>
 
 #include <string.h>
 
@@ -42,8 +43,6 @@ enum {
 	PROP_PROVIDER,
 };
 
-static void  update_properties   (RealmSssd *self);
-
 G_DEFINE_TYPE (RealmSssd, realm_sssd, REALM_TYPE_KERBEROS);
 
 static void
@@ -57,57 +56,59 @@ on_logins_restarted (GObject *source,
                      GAsyncResult *result,
                      gpointer user_data)
 {
-	GSimpleAsyncResult *async = G_SIMPLE_ASYNC_RESULT (user_data);
-	RealmSssd *self = REALM_SSSD (g_async_result_get_source_object (user_data));
+	GTask *task = G_TASK (user_data);
+	RealmSssd *self = g_task_get_source_object (task);
 	GError *error = NULL;
 
 	realm_service_restart_finish (result, &error);
-	if (error != NULL)
-		g_simple_async_result_take_error (async, error);
-
-	update_properties (self);
-	g_simple_async_result_complete (async);
-
-	g_object_unref (async);
-	g_object_unref (self);
+	if (error != NULL) {
+		g_task_return_error (task, error);
+	} else {
+		realm_sssd_update_properties (self);
+		g_task_return_boolean (task, TRUE);
+	}
+	g_object_unref (task);
 }
 
-static gboolean
-sssd_config_change_login_policy (RealmIniConfig *config,
-                                 const gchar *section,
-                                 const gchar *access_provider,
-                                 const gchar **add_names,
-                                 const gchar **remove_names,
-                                 GError **error)
+gboolean
+realm_sssd_set_login_policy (RealmIniConfig *config,
+                             const gchar *section,
+                             const gchar *access_provider,
+                             const gchar **add_names,
+                             const gchar **remove_names,
+                             gboolean names_are_groups,
+                             GError **error)
 {
-	gchar *allow;
+	const gchar *field = names_are_groups ? "simple_allow_groups" : "simple_allow_users";
+	gchar *allow = NULL;
 
 	if (!realm_ini_config_begin_change (config, error))
 		return FALSE;
 
 	if (access_provider)
-		realm_ini_config_set (config, section, "access_provider", access_provider);
-	realm_ini_config_set_list_diff (config, section, "simple_allow_users", ",",
-	                                add_names, remove_names);
+		realm_ini_config_set (config, section, "access_provider", access_provider, NULL);
 
-	/*
-	 * HACK: Work around for sssd problem where it allows users if
-	 * simple_allow_users is empty. Set it to a comma in this case.
-	 */
-	allow = realm_ini_config_get (config, section, "simple_allow_users");
-	if (allow != NULL) {
-		g_strstrip (allow);
-		if (g_str_equal (allow, "") || g_str_equal (allow, ",")) {
-			g_free (allow);
-			allow = NULL;
+	if (!access_provider || g_str_equal (access_provider, "simple")) {
+		realm_ini_config_set_list_diff (config, section, field, ",", add_names, remove_names);
+
+		/*
+		 * HACK: Work around for sssd problem where it allows users if
+		 * simple_allow_users is empty. Set it to a dollar in this case.
+		 */
+		allow = realm_ini_config_get (config, section, field);
+		if (allow != NULL) {
+			g_strstrip (allow);
+			if (g_str_equal (allow, "") || g_str_equal (allow, "$") || g_str_equal (allow, ",")) {
+				g_free (allow);
+				allow = NULL;
+			}
 		}
-	}
 
-	if (allow == NULL) {
-		if (g_str_equal (access_provider, "simple"))
-			realm_ini_config_set (config, section, "simple_allow_users", ",");
-		else
-			realm_ini_config_set (config, section, "simple_allow_users", NULL);
+		if (allow == NULL)
+			realm_ini_config_set (config, section, field, "$", NULL);
+	} else {
+		realm_ini_config_set (config, section, "simple_allow_users", NULL, NULL);
+		realm_ini_config_set (config, section, "simple_allow_groups", NULL, NULL);
 	}
 
 	g_free (allow);
@@ -115,35 +116,54 @@ sssd_config_change_login_policy (RealmIniConfig *config,
 	return realm_ini_config_finish_change (config, error);
 }
 
+static gboolean
+sssd_config_check_login_list (const gchar **logins,
+                              GError **error)
+{
+	#define INVALID_CHARS ",$"
+	gint i;
+
+	for (i = 0; logins != NULL && logins[i] != NULL; i++) {
+		if (strcspn (logins[i], INVALID_CHARS) != strlen (logins[i])) {
+			g_set_error (error, G_DBUS_ERROR,
+			             G_DBUS_ERROR_INVALID_ARGS,
+			             _("Invalid login argument '%s' contains unsupported characters."),
+			             logins[i]);
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
 static void
 realm_sssd_logins_async (RealmKerberos *realm,
                          GDBusMethodInvocation *invocation,
                          RealmKerberosLoginPolicy login_policy,
                          const gchar **add,
                          const gchar **remove,
+                         GVariant *options,
                          GAsyncReadyCallback callback,
                          gpointer user_data)
 {
+	RealmSssdClass *sssd_class = REALM_SSSD_GET_CLASS (realm);
 	RealmSssd *self = REALM_SSSD (realm);
-	GSimpleAsyncResult *async;
+	gboolean names_are_groups = FALSE;
+	GTask *task;
 	gchar **remove_names = NULL;
 	gchar **add_names = NULL;
-	gboolean ret = FALSE;
 	GError *error = NULL;
 	const gchar *access_provider;
 
+	task = g_task_new (realm, NULL, callback, user_data);
+
 	if (!self->pv->section) {
-		async = g_simple_async_result_new_error (G_OBJECT (realm), callback, user_data,
-		                                         REALM_ERROR, REALM_ERROR_NOT_CONFIGURED,
-		                                         "Not joined to this domain");
-		g_simple_async_result_complete_in_idle (async);
-		g_object_unref (async);
+		g_task_return_new_error (task, REALM_ERROR, REALM_ERROR_NOT_CONFIGURED,
+		                         "Not joined to this domain");
+		g_object_unref (task);
 		return;
 	}
 
-	async = g_simple_async_result_new (G_OBJECT (realm), callback, user_data,
-	                                   realm_sssd_logins_async);
-
 	switch (login_policy) {
 	case REALM_KERBEROS_POLICY_NOT_SET:
 		access_provider = NULL;
@@ -151,6 +171,9 @@ realm_sssd_logins_async (RealmKerberos *realm,
 	case REALM_KERBEROS_ALLOW_ANY_LOGIN:
 		access_provider = "permit";
 		break;
+	case REALM_KERBEROS_ALLOW_REALM_LOGINS:
+		access_provider = sssd_class->sssd_conf_provider_name;
+		break;
 	case REALM_KERBEROS_ALLOW_PERMITTED_LOGINS:
 		access_provider = "simple";
 		break;
@@ -161,37 +184,43 @@ realm_sssd_logins_async (RealmKerberos *realm,
 		g_return_if_reached ();
 	}
 
-	add_names = realm_kerberos_parse_logins (realm, TRUE, add, &error);
-	if (add_names != NULL)
-		remove_names = realm_kerberos_parse_logins (realm, TRUE, remove, &error);
+	if (!g_variant_lookup (options, "groups", "b", &names_are_groups))
+		names_are_groups = FALSE;
 
-	if (add_names == NULL || remove_names == NULL) {
-		g_simple_async_result_take_error (async, error);
-		g_simple_async_result_complete_in_idle (async);
-		g_object_unref (async);
-		return;
+	if (!names_are_groups) {
+		add_names = realm_kerberos_parse_logins (realm, TRUE, add, &error);
+		if (add_names != NULL)
+			remove_names = realm_kerberos_parse_logins (realm, TRUE, remove, &error);
+		add = (const gchar **)add_names;
+		remove = (const gchar **)remove_names;
 	}
 
-	ret = sssd_config_change_login_policy (self->pv->config,
-	                                       self->pv->section,
-	                                       access_provider,
-	                                       (const gchar **)add_names,
-	                                       (const gchar **)remove_names,
-	                                       &error);
+	if (error == NULL)
+		sssd_config_check_login_list (add, &error);
+	if (error == NULL)
+		sssd_config_check_login_list (remove, &error);
 
-	if (ret) {
+	if (error == NULL) {
+		realm_sssd_set_login_policy (self->pv->config,
+		                             self->pv->section,
+		                             access_provider,
+		                             add, remove,
+		                             names_are_groups,
+		                             &error);
+	}
+
+	if (error == NULL) {
 		realm_service_restart ("sssd", invocation,
 		                       on_logins_restarted,
-		                       g_object_ref (async));
+		                       g_object_ref (task));
 	} else {
-		g_simple_async_result_take_error (async, error);
-		g_simple_async_result_complete_in_idle (async);
+		g_task_return_error (task, error);
 	}
 
 	g_strfreev (remove_names);
 	g_strfreev (add_names);
 
-	g_object_unref (async);
+	g_object_unref (task);
 }
 
 static gboolean
@@ -199,29 +228,41 @@ realm_sssd_generic_finish (RealmKerberos *realm,
                            GAsyncResult *result,
                            GError **error)
 {
-	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
-		return FALSE;
-
-	return TRUE;
+	return g_task_propagate_boolean (G_TASK (result), error);
 }
 
 static void
-update_enrolled (RealmSssd *self)
+update_configured (RealmSssd *self)
 {
+	gboolean manages_system;
+	gchar *value;
+
 	realm_kerberos_set_configured (REALM_KERBEROS (self),
 	                               self->pv->section ? TRUE : FALSE);
+
+	manages_system = FALSE;
+	if (self->pv->section) {
+		value = realm_ini_config_get (self->pv->config, self->pv->section, "realmd_tags");
+		if (value && strstr (value, "manages-system"))
+			manages_system = TRUE;
+		g_free (value);
+	}
+
+	realm_kerberos_set_manages_system (REALM_KERBEROS (self), manages_system);
 }
 
-static void
-update_realm_name (RealmSssd *self)
+static gchar *
+calc_realm_name (RealmSssd *self)
 {
 	RealmKerberos *kerberos = REALM_KERBEROS (self);
 	const char *name;
+	RealmDisco *disco;
 	gchar *realm = NULL;
 
 	if (self->pv->section == NULL) {
-		realm = g_strdup (realm_discovery_get_string (realm_kerberos_get_discovery (kerberos),
-		                                              REALM_DBUS_DISCOVERY_REALM));
+		disco = realm_kerberos_get_disco (kerberos);
+		if (disco != NULL)
+			realm = g_strdup (disco->kerberos_realm);
 	} else {
 		realm = realm_ini_config_get (self->pv->config, self->pv->section, "krb5_realm");
 	}
@@ -231,20 +272,29 @@ update_realm_name (RealmSssd *self)
 		realm = name ? g_ascii_strup (name, -1) : NULL;
 	}
 
-	realm_kerberos_set_realm_name (kerberos, realm);
-	g_free (realm);
+	return realm;
 }
 
 static void
-update_domain (RealmSssd *self)
+update_realm_name (RealmSssd *self)
+{
+	gchar *realm = calc_realm_name (self);
+	realm_kerberos_set_realm_name (REALM_KERBEROS (self), realm);
+	g_free (realm);
+}
+
+static gchar *
+calc_domain (RealmSssd *self)
 {
 	RealmKerberos *kerberos = REALM_KERBEROS (self);
 	const char *name;
+	RealmDisco *disco;
 	gchar *domain = NULL;
 
 	if (self->pv->section == NULL) {
-		domain = g_strdup (realm_discovery_get_string (realm_kerberos_get_discovery (kerberos),
-		                                               REALM_DBUS_DISCOVERY_DOMAIN));
+		disco = realm_kerberos_get_disco (kerberos);
+		if (disco != NULL)
+			domain = g_strdup (disco->domain_name);
 	} else {
 		domain = realm_ini_config_get (self->pv->config, self->pv->section, "dns_discovery_domain");
 	}
@@ -254,75 +304,104 @@ update_domain (RealmSssd *self)
 		domain = name ? g_ascii_strdown (name, -1) : NULL;
 	}
 
-	realm_kerberos_set_domain_name (kerberos, domain);
+	return domain;
+}
+
+static void
+update_domain (RealmSssd *self)
+{
+	gchar *domain = calc_domain (self);
+	realm_kerberos_set_domain_name (REALM_KERBEROS (self), domain);
 	g_free (domain);
 }
 
-static gchar *
-build_login_format (const gchar *format,
-                    ...)
-{
-	gchar *result;
-	va_list va;
-
-	/* This function exists mostly to get around gcc warnings */
-
-	if (format == NULL)
-		format = "%1$s@%2$s";
-
-	va_start (va, format);
-	result = g_strdup_vprintf (format, va);
-	va_end (va);
-
-	return result;
-}
-
-
 static void
 update_login_formats (RealmSssd *self)
 {
 	RealmKerberos *kerberos = REALM_KERBEROS (self);
 	gchar *login_formats[2] = { NULL, NULL };
 	gchar *format = NULL;
+	gchar *domain_name;
+	gboolean qualify;
 
 	if (self->pv->section == NULL) {
 		realm_kerberos_set_login_formats (kerberos, (const gchar **)login_formats);
 		return;
 	}
 
+	qualify = realm_ini_config_get_boolean (self->pv->config, self->pv->section,
+	                                        "use_fully_qualified_names", FALSE);
+
+	if (!qualify) {
+		login_formats[0] = "%U";
+		realm_kerberos_set_login_formats (kerberos, (const gchar **)login_formats);
+		return;
+	}
+
 	/* Setup the login formats */
 	format = realm_ini_config_get (self->pv->config, self->pv->section, "full_name_format");
 
-	/* Here we place a '%s' in the place of the user in the format */
-	login_formats[0] = build_login_format (format, "%U", self->pv->domain);
-	realm_kerberos_set_login_formats (kerberos, (const gchar **)login_formats);
+	/* The full domain name */
+	domain_name = calc_domain (self);
+
+	/*
+	 * In theory we should be discovering the short name or flat name as sssd
+	 * calls it. We configured it as the sssd.conf 'domains' name, so we just
+	 * use that. Eventually we want to have a way to query sssd for that.
+	 */
+
+	/*
+	 * Here we place a '%U' in the place of the user in the format, and
+	 * fill in the domain appropriately. sssd uses snprintf for this, which
+	 * is risky and very compex to do right with positional arguments.
+	 *
+	 * We only replace the arguments documented in sssd.conf, as well as
+	 * other non-field printf replacements.
+	 */
+
+	if (safe_asprintf (login_formats, format, "%U", domain_name ? domain_name : "", self->pv->domain, NULL) >= 0)
+		realm_kerberos_set_login_formats (kerberos, (const gchar **)login_formats);
 	g_free (login_formats[0]);
+	g_free (domain_name);
 	g_free (format);
 }
 
+#pragma GCC diagnostic pop
+
 static void
 update_login_policy (RealmSssd *self)
 {
+	RealmSssdClass *sssd_class = REALM_SSSD_GET_CLASS (self);
 	RealmKerberosLoginPolicy policy = REALM_KERBEROS_POLICY_NOT_SET;
 	RealmKerberos *kerberos = REALM_KERBEROS (self);
-	GPtrArray *permitted;
+	GPtrArray *permitted_logins;
+	GPtrArray *permitted_groups;
 	gchar *access = NULL;
 	gchar **values;
 	gint i;
 
-	permitted = g_ptr_array_new_full (0, g_free);
+	permitted_logins = g_ptr_array_new_full (0, g_free);
+	permitted_groups = g_ptr_array_new_full (0, g_free);
 	if (self->pv->section != NULL)
 		access = realm_ini_config_get (self->pv->config, self->pv->section, "access_provider");
 	if (g_strcmp0 (access, "simple") == 0) {
 		values = realm_ini_config_get_list (self->pv->config, self->pv->section,
 		                                    "simple_allow_users", ",");
 		for (i = 0; values != NULL && values[i] != NULL; i++) {
-			if (!g_str_equal (values[i], ""))
-				g_ptr_array_add (permitted, realm_kerberos_format_login (kerberos, values[i]));
+			if (!g_str_equal (values[i], "") && !g_str_equal (values[i], "$"))
+				g_ptr_array_add (permitted_logins, realm_kerberos_format_login (kerberos, values[i]));
+		}
+		g_strfreev (values);
+		values = realm_ini_config_get_list (self->pv->config, self->pv->section,
+		                                    "simple_allow_groups", ",");
+		for (i = 0; values != NULL && values[i] != NULL; i++) {
+			if (!g_str_equal (values[i], "") && !g_str_equal (values[i], "$"))
+				g_ptr_array_add (permitted_groups, g_strdup (values[i]));
 		}
 		g_strfreev (values);
-		g_free (access);
 		policy = REALM_KERBEROS_ALLOW_PERMITTED_LOGINS;
+	} else if (g_strcmp0 (access, sssd_class->sssd_conf_provider_name) == 0) {
+		policy = REALM_KERBEROS_ALLOW_REALM_LOGINS;
 	} else if (g_strcmp0 (access, "permit") == 0) {
 		policy = REALM_KERBEROS_ALLOW_ANY_LOGIN;
 	} else if (g_strcmp0 (access, "deny") == 0) {
@@ -331,50 +410,55 @@ update_login_policy (RealmSssd *self)
 		policy = REALM_KERBEROS_POLICY_NOT_SET;
 	}
 
-	g_ptr_array_add (permitted, NULL);
+	g_ptr_array_add (permitted_logins, NULL);
+	g_ptr_array_add (permitted_groups, NULL);
 
 	realm_kerberos_set_login_policy (kerberos, policy);
-	realm_kerberos_set_permitted_logins (kerberos, (const gchar **)permitted->pdata);
+	realm_kerberos_set_permitted_logins (kerberos, (const gchar **)permitted_logins->pdata);
+	realm_kerberos_set_permitted_groups (kerberos, (const gchar **)permitted_groups->pdata);
 
-	g_ptr_array_free (permitted, TRUE);
+	g_ptr_array_free (permitted_logins, TRUE);
+	g_ptr_array_free (permitted_groups, TRUE);
+	g_free (access);
 }
 
-static void
-update_properties (RealmSssd *self)
+void
+realm_sssd_update_properties (RealmSssd *self)
 {
 	GObject *obj = G_OBJECT (self);
-	const gchar *name;
+	const gchar *my_name;
+	gchar *name = NULL;
 	gchar *section = NULL;
-	gchar *domain = NULL;
 	gchar **domains;
-	gchar *realm;
 	gint i;
 
 	g_object_freeze_notify (obj);
 
+	g_free (self->pv->section);
+	self->pv->section = NULL;
+
+	g_free (self->pv->domain);
+	self->pv->domain = NULL;
+
 	/* Find the config domain with our realm */
 	domains = realm_sssd_config_get_domains (self->pv->config);
-	name = realm_kerberos_get_name (REALM_KERBEROS (self));
-	for (i = 0; domains && domains[i]; i++) {
-		section = realm_sssd_config_domain_to_section (domains[i]);
-		realm = realm_ini_config_get (self->pv->config, section, "krb5_realm");
-		if (realm && name && g_ascii_strcasecmp (realm, name) == 0) {
-			domain = g_strdup (domains[i]);
-			break;
-		} else {
+	my_name = realm_kerberos_get_name (REALM_KERBEROS (self));
+	for (i = 0; self->pv->section == NULL && domains && domains[i]; i++) {
+		if (realm_sssd_config_load_domain (self->pv->config, domains[i], &section, NULL, &name)) {
+			if (my_name && name && g_ascii_strcasecmp (my_name, name) == 0) {
+				self->pv->domain = g_strdup (domains[i]);
+				self->pv->section = section;
+				section = NULL;
+			}
+
 			g_free (section);
-			section = NULL;
+			g_free (name);
 		}
 	}
 	g_strfreev (domains);
 
-	g_free (self->pv->section);
-	self->pv->section = section;
-	g_free (self->pv->domain);
-	self->pv->domain = domain;
-
 	/* Update all the other properties */
-	update_enrolled (self);
+	update_configured (self);
 	update_realm_name (self);
 	update_domain (self);
 	update_login_formats (self);
@@ -387,7 +471,7 @@ static void
 on_config_changed (RealmIniConfig *config,
                    gpointer user_data)
 {
-	update_properties (REALM_SSSD (user_data));
+	realm_sssd_update_properties (REALM_SSSD (user_data));
 }
 
 static void
@@ -418,7 +502,7 @@ realm_sssd_notify (GObject *obj,
                    GParamSpec *spec)
 {
 	if (g_str_equal (spec->name, "name"))
-		update_properties (REALM_SSSD (obj));
+		realm_sssd_update_properties (REALM_SSSD (obj));
 
 	if (G_OBJECT_CLASS (realm_sssd_parent_class)->notify)
 		G_OBJECT_CLASS (realm_sssd_parent_class)->notify (obj, spec);
@@ -494,7 +578,7 @@ realm_sssd_build_default_home (const gchar *value)
 }
 
 typedef struct {
-	GSimpleAsyncResult *async;
+	GTask *task;
 	GDBusMethodInvocation *invocation;
 	RealmIniConfig *config;
 	gchar *domain;
@@ -504,11 +588,11 @@ static void
 deconfigure_closure_free (gpointer data)
 {
 	DeconfClosure *deconf = data;
-	g_object_unref (deconf->async);
+	g_object_unref (deconf->task);
 	g_object_unref (deconf->invocation);
 	g_object_unref (deconf->config);
 	g_free (deconf->domain);
-	g_slice_free (DeconfClosure, deconf);
+	g_free (deconf);
 }
 
 static void
@@ -525,7 +609,7 @@ on_service_disable_done (GObject *source,
 		g_error_free (error);
 	}
 
-	g_simple_async_result_complete (deconf->async);
+	g_task_return_boolean (deconf->task, TRUE);
 	deconfigure_closure_free (deconf);
 }
 
@@ -543,7 +627,7 @@ on_service_restart_done (GObject *source,
 		g_error_free (error);
 	}
 
-	g_simple_async_result_complete (deconf->async);
+	g_task_return_boolean (deconf->task, TRUE);
 	deconfigure_closure_free (deconf);
 }
 
@@ -584,20 +668,21 @@ on_sssd_clear_cache (GObject *source,
 		g_clear_error (&error);
 	}
 
-	/* Deconfigure sssd.conf */
-	realm_diagnostics_info (deconf->invocation, "Removing domain configuration from sssd.conf");
-	if (!realm_sssd_config_remove_domain (deconf->config, deconf->domain, &error)) {
-		g_simple_async_result_take_error (deconf->async, error);
-		g_simple_async_result_complete (deconf->async);
-		deconfigure_closure_free (deconf);
-		return;
+	/* Deconfigure sssd.conf, may have already been done, if so NULL */
+	if (deconf->domain) {
+		realm_diagnostics_info (deconf->invocation, "Removing domain configuration from sssd.conf");
+		if (!realm_sssd_config_remove_domain (deconf->config, deconf->domain, &error)) {
+			g_task_return_error (deconf->task, error);
+			deconfigure_closure_free (deconf);
+			return;
+		}
 	}
 
 	/* If no domains, then disable sssd */
 	domains = realm_sssd_config_get_domains (deconf->config);
 	if (domains == NULL || g_strv_length (domains) == 0) {
 		realm_command_run_known_async ("sssd-disable-logins", NULL, deconf->invocation,
-		                               NULL, on_disable_nss_service, deconf);
+		                               on_disable_nss_service, deconf);
 
 	/* If any domains left, then restart sssd */
 	} else {
@@ -610,7 +695,7 @@ on_sssd_clear_cache (GObject *source,
 
 void
 realm_sssd_deconfigure_domain_tail (RealmSssd *self,
-                                    GSimpleAsyncResult *async,
+                                    GTask *task,
                                     GDBusMethodInvocation *invocation)
 {
 	DeconfClosure *deconf;
@@ -622,13 +707,12 @@ realm_sssd_deconfigure_domain_tail (RealmSssd *self,
 	/* Flush the keytab of all the entries for this realm */
 	realm_diagnostics_info (invocation, "Removing entries from keytab for realm");
 	if (!realm_kerberos_flush_keytab (realm_name, &error)) {
-		g_simple_async_result_take_error (async, error);
-		g_simple_async_result_complete_in_idle (async);
+		g_task_return_error (task, error);
 		return;
 	}
 
 	deconf = g_new0 (DeconfClosure, 1);
-	deconf->async = g_object_ref (async);
+	deconf->task = g_object_ref (task);
 	deconf->invocation = g_object_ref (invocation);
 	deconf->config = g_object_ref (self->pv->config);
 	deconf->domain = g_strdup (self->pv->domain);
@@ -639,5 +723,5 @@ realm_sssd_deconfigure_domain_tail (RealmSssd *self,
 	 */
 
 	realm_command_run_known_async ("sssd-caches-flush", NULL, deconf->invocation,
-	                               NULL, on_sssd_clear_cache, deconf);
+	                               on_sssd_clear_cache, deconf);
 }
